from OpenGL.GL import *
import math, body, vector

class Nautilus(body.Body):
    def __init__(self, mesh):
        body.Body.__init__(self)
        self.mesh = mesh
        self.steering_angle = 0
        self.steering_target_angle = 0
        self.propeller_angle = 0
        self.propeller_speed = 0
        self.rotor_angle = 0
        self.air = 3600
        self.score = 0

    def auto_level(self):
        if abs(self.b.y) > 0.9: return
        up = vector.Vector(0, 1, 0)
        c = up.dot(self.r)
        a = math.acos(c)
        if a < math.pi / 2: self.turn(0, 0, -0.0001)
        elif a > math.pi / 2: self.turn(0, 0, 0.0001)

    def tick(self):
        body.Body.tick(self)
        self.propeller_angle += self.propeller_speed
        self.propeller_speed *= 0.98
        if abs(self.propeller_speed) < 0.1: self.propeller_speed = 0

        da = self.steering_target_angle - self.steering_angle
        if da > 180: da -= 360
        if da < -180: da += 360
        self.steering_angle += da * 0.1
        if self.steering_angle < 0: self.steering_angle += 360
        if self.steering_angle > 360: self.steering_angle -= 360

        self.rotor_angle += self.engine * 10000

        self.auto_level()

        self.air -= 1

    def render(self):
        def trans(p): glTranslatef(p.x, p.y, p.z)
        def retrans(p): glTranslatef(-p.x, -p.y, -p.z)

        # Main hull
        glPushMatrix()
        glTranslatef(self.position.x, self.position.y, self.position.z)
        matrix = self.matrix()
        glMultMatrixf(matrix)
        self.mesh.render()

        # Steering
        glPushMatrix()
        trans(self.steering_point)
        glRotatef(self.steering_angle, 0, 0, 1)
        retrans(self.steering_point)
        self.steering.render()

        # Steering propellor
        trans(self.propeller_point)
        glRotatef(self.propeller_angle, 1, 0, 0)
        retrans(self.propeller_point)
        self.propeller.render()
        glPopMatrix()

        # Left rotor
        glPushMatrix()
        trans(self.left_point)
        glRotatef(self.rotor_angle, 0, 0, 1)
        retrans(self.left_point)
        self.left_rotor.render()
        glPopMatrix()

        # Right rotor
        glPushMatrix()
        trans(self.right_point)
        glRotatef(self.rotor_angle, 0, 0, 1)
        retrans(self.right_point)
        self.right_rotor.render()
        glPopMatrix()

        glPopMatrix()


