from OpenGL.GL import *
import exceptions, struct

"""
Loader for the "swarm" mesh format.
"""

class Mesh:
    def __init__(self):
        self.vertices = []
        self.triangles = []
        self.display_list = None

    def render(self):
        if self.display_list == None:
            self.display_list = glGenLists(1)
            glNewList(self.display_list, GL_COMPILE_AND_EXECUTE)
            glBegin(GL_TRIANGLES)
            for t in self.triangles:
                glColor4f(t.r, t.g, t.b, t.a)
                v1 = self.vertices[t.v1]
                v2 = self.vertices[t.v2]
                v3 = self.vertices[t.v3]
                glNormal3f(v1.nx, v1.ny, v1.nz)
                glVertex3f(v1.x, v1.y, v1.z)
                glNormal3f(v2.nx, v2.ny, v2.nz)
                glVertex3f(v2.x, v2.y, v2.z)
                glNormal3f(v3.nx, v3.ny, v3.nz)
                glVertex3f(v3.x, v3.y, v3.z)
            glEnd()
            glEndList()
        else:
            glCallList(self.display_list)

class Vertex:
    def __init__(self, x, y, z, nx, ny, nz):
        self.x = x
        self.y = y
        self.z = z
        self.nx = nx
        self.ny = ny
        self.nz = nz

class Triangle:
    def __init__(self, v1, v2, v3, r, g, b, a):
        self.v1 = v1
        self.v2 = v2
        self.v3 = v3
        self.r = r
        self.g = g
        self.b = b
        self.a = a

def read_str(f):
    str = ""
    while True:
        c = f.read(1)
        if c == "": break
        if c == "\0": break;
        str += c
    return str

def getd(f): return struct.unpack("<d", f.read(8))[0]
def geti(f): return struct.unpack("<i", f.read(4))[0]

class SwarmException(exceptions.Exception):
    def __init__(self, str):
        self.str = str
    def __str__(self):
        return self.str

def load_meshes(filename):
    n = 0
    f = file(filename)
    str = read_str(f)
    if str != "SWARMFILE":
        raise SwarmException("%s is not a swarm file, got «%s»!" % (filename,
            str))
    meshes = {}
    while True:
        is_mesh = 0
        is_empty = 0
        str = read_str(f);
        if str == "mesh":
            is_mesh = 1;
        elif str == "empty":
            is_empty = 1
        else:
            break
        name = read_str(f);
        mesh = Mesh()
        vn = geti(f)
        tn = geti(f)
        print name, vn, tn
        meshes[name] = mesh
        for i in range(vn):
            x = getd(f)
            y = getd(f)
            z = getd(f)
            nx = getd(f)
            ny = getd(f)
            nz = getd(f)
            v = Vertex(x, y, z, nx, ny, nz)
            mesh.vertices.append(v)

        for i in range(tn):
            v1 = geti(f)
            v2 = geti(f)
            v3 = geti(f)
            r = getd(f)
            g = getd(f)
            b = getd(f)
            a = getd(f)
            t = Triangle(v1, v2, v3, r, g, b, a)
            mesh.triangles.append(t)

    return meshes
