#!/usr/bin/env python
# encoding: utf8
import pygame, math, random
from OpenGL.GL import *
import body, gauges
from vector import Vector
from nautilus import Nautilus
from camera import Camera
import swarm, data

def lighting():
    glEnable(GL_LIGHTING)
    glEnable(GL_COLOR_MATERIAL)
    glEnable(GL_BLEND)

    glMaterialfv(GL_FRONT, GL_AMBIENT, (0, 0.1, 0.1, 1));

    glMaterialfv(GL_FRONT, GL_EMISSION, (0, 0, 0, 1));
    glMaterialfv(GL_FRONT, GL_SPECULAR, (1, 1, 1, 1));
    glMaterialf(GL_FRONT, GL_SHININESS, 1);

    glColorMaterial(GL_FRONT, GL_DIFFUSE)

    glLightModel(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE)
    glLightModel(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE)

    glEnable(GL_LIGHT0)
    glLightfv(GL_LIGHT0, GL_AMBIENT, (1, 1, 1, 1));
    glLightfv(GL_LIGHT0, GL_SPECULAR, (0, 0, 0, 0));
    glLightfv(GL_LIGHT0, GL_DIFFUSE, (1, 1, 1, 1));
    glLightfv(GL_LIGHT0, GL_POSITION, (0, 21000, 0, 1));

    glEnable(GL_LIGHT1)
    glLightfv(GL_LIGHT1, GL_AMBIENT, (0, 0, 0, 0));
    glLightfv(GL_LIGHT1, GL_SPECULAR, (0, 0, 0, 0));
    glLightfv(GL_LIGHT1, GL_DIFFUSE, (0.3, 0.2, 0.1, 0));
    glLightfv(GL_LIGHT1, GL_POSITION, (0, -1000, 0, 1));

class Buoy:
    def __init__(self, mesh, position):
        self.mesh = mesh
        self.position = position

def spiral(y):
    for i in range(10):
        a = math.pi * i / 5
        c = math.cos(a)
        s = math.sin(a)
        r = 500
        y += 100
        yield -r * c, y, -r * s
    for i in range(10):
        a = -math.pi * i / 5
        c = math.cos(a)
        s = math.sin(a)
        r = 500
        y += 100
        yield -r * c, y, -r * s

def level2(meshes):
    global buoys
    buoys = []
    for x, y, z in spiral(2000):
        buoys.append(Buoy(meshes["Buoy"], Vector(x, y, z)))

def level1(meshes):
    global buoys
    buoys = []
    y = 0
    x = -500
    z = 500
    
    for i in range(5):
        y += 100
        z -= 200
        buoys.append(Buoy(meshes["Buoy"], Vector(x, y, z)))
    for i in range(5):
        y += 100
        x += 200
        buoys.append(Buoy(meshes["Buoy"], Vector(x, y, z)))
    for i in range(5):
        y += 100
        z += 200
        buoys.append(Buoy(meshes["Buoy"], Vector(x, y, z)))
    for i in range(5):
        y += 100
        x -= 200
        buoys.append(Buoy(meshes["Buoy"], Vector(x, y, z)))

def main():
    w = 320
    h = 240
    FPS = 50

    pygame.init()
    pygame.display.set_mode((w * 2, h * 2), pygame.OPENGL | pygame.DOUBLEBUF |
        pygame.FULLSCREEN)

    pygame.mouse.set_visible(False)

    ws = w / 800.0
    hs = h / 800.0

    fov = 45
    angle = fov * math.pi / 180.0 / 2
    near = ws / math.tan(angle)

    camera = Camera()
    camera.w = ws
    camera.h = hs
    camera.near = near

    meshes = swarm.load_meshes(data.filepath("nautilus.swarm"))

    nautilus = Nautilus(meshes["Nautilus"])
    nautilus.engine = 0
    nautilus.steering = meshes["Steering"]
    nautilus.propeller = meshes["Propeller"]
    nautilus.left_rotor = meshes["Rotor.L"]
    nautilus.right_rotor = meshes["Rotor.R"]

    level1(meshes)

    nautilus.position.x = buoys[0].position.x
    nautilus.position.y = 0
    nautilus.position.z = buoys[0].position.z + 200

    nautilus.ENGINE_MAX = 0.005

    v = meshes["PropellerPoint"].vertices[0]
    nautilus.propeller_point = Vector(v.x, v.y, v.z)

    v = meshes["SteeringPoint"].vertices[0]
    nautilus.steering_point = Vector(v.x, v.y, v.z)

    v = meshes["RotorPoint.L"].vertices[0]
    nautilus.left_point = Vector(v.x, v.y, v.z)

    v = meshes["RotorPoint.R"].vertices[0]
    nautilus.right_point = Vector(v.x, v.y, v.z)

    v = meshes["ScopePoint"].vertices[0]
    nautilus.scope_point = Vector(v.x, v.y, v.z)

    glMatrixMode(GL_PROJECTION)
    glFrustum(-ws, ws, -hs, hs, near, 2000)
    glMatrixMode(GL_MODELVIEW)

    clock = pygame.time.Clock()

    left = False
    right = False
    up = False
    down = False
    accel = False
    reverse = False

    glEnable(GL_BLEND)
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
    glEnable(GL_LINE_SMOOTH)

    # block of stars for testing
    displaylist = glGenLists(1)
    glNewList(displaylist, GL_COMPILE)
    glBegin(GL_POINTS)
    for i in range(256):
        x = (random.random() - 0.5) * 1000
        y = 20000
        z = (random.random() - 0.5) * 1000
        glColor4f(1, 1, 1, 1)
        glVertex3f(x, y, z)
    glEnd()
    glEndList();

    #r = 1.0
    #g = 1.0
    #b = 0.0

    level = 1
    show_level = 300

    pygame.mouse.get_rel()

    show_help = True
    quit = False
    next = pygame.time.get_ticks() + 1000 / FPS
    while not quit:
        zoom = 0
        for e in pygame.event.get():
            if e.type == pygame.QUIT: quit = True
            elif e.type in [pygame.KEYDOWN, pygame.KEYUP]:
                on = e.type == pygame.KEYDOWN
                if e.key == pygame.K_ESCAPE: quit = on
                if e.key in [pygame.K_LEFT, pygame.K_a]: left = on
                if e.key in [pygame.K_RIGHT, pygame.K_d]: right = on
                if e.key in [pygame.K_UP, pygame.K_w]: up = on
                if e.key in [pygame.K_DOWN,  pygame.K_s]: down = on
                if e.key in [pygame.K_SPACE, pygame.K_RSHIFT, pygame.K_LSHIFT]:
                    accel = on
                if e.key in [pygame.K_BACKSPACE, pygame.K_RCTRL,
                    pygame.K_LCTRL]: reverse = on

                if e.key == pygame.K_F1 and on:
                    show_help = not show_help
                if e.key == pygame.K_F10 and on:
                    pygame.event.set_grab(not pygame.event.get_grab())
                if e.key == pygame.K_RETURN and on:
                    pygame.display.toggle_fullscreen()
            elif e.type == pygame.MOUSEBUTTONUP:
                if e.button == 1: accel = 0
                if e.button == 3: reverse = 0
                if e.button == 4: zoom = -1
                if e.button == 5: zoom = 1
            elif e.type == pygame.MOUSEBUTTONDOWN:
                if e.button == 1: accel = 1
                if e.button == 3: reverse = 1

        if show_level: show_level -= 1

        steering = [0, 0]
        if up:
            nautilus.turn(0.0005, 0, 0)
            steering[1] = -1
        if down:
            nautilus.turn(-0.0005, 0, 0)
            steering[1] = 1

        if accel: nautilus.engine -= 0.0001
        if reverse: nautilus.engine += 0.0001

        m = nautilus.ENGINE_MAX
        if nautilus.engine < -m: nautilus.engine = -m
        if nautilus.engine > m: nautilus.engine = m
        
        if left:
            nautilus.turn(0, 0.0005, 0)
            steering[0] = 1
        if right:
            nautilus.turn(0, -0.0005, 0)
            steering[0] = -1

        if steering[0] or steering[1]:
            nautilus.propeller_speed = 20
            a = math.atan2(steering[1], steering[0])
            nautilus.steering_target_angle = a * 180 / math.pi
            if nautilus.steering_target_angle < 0:
                nautilus.steering_target_angle += 360

        nautilus.move(0, 0, nautilus.engine)

        p = nautilus.position
        r = math.sqrt(p.x ** 2 + p.z ** 2)
        if r > 1000:
            nautilus.move(0.01 * -p.x / r, -0.01, 0.01 * -p.z / r)

        if p.y < -100: nautilus.move(0, 0.02, 0)
        if p.y > 20000: nautilus.move(0, -0.02, 0)

        mx, my = pygame.mouse.get_rel()

        camera.change(my * 0.005, mx * 0.005, zoom * 0.5)

        b = buoys[0]
        d = (b.position - nautilus.position).norm()
        if d < 4:
            buoys.pop(0)
            nautilus.score += nautilus.air
            nautilus.air = 3600

        # gravity
        # nautilus.move(0, -0.1, 0)

        nautilus.damp()
        nautilus.tick()

        glDisable(GL_TEXTURE_2D)
        glEnable(GL_DEPTH_TEST)
	glEnable(GL_CULL_FACE)
	glShadeModel(GL_SMOOTH)

        x = -camera.b.y * 0.5
        if x < 0: x = 0
        glClearColor(x * 0.2, x * 0.5, x, 1)
        glClearDepth(1);
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

        camera.recalculate(nautilus.position + nautilus.scope_point)
        camera.view()

        glCallList(displaylist)

        lighting()

        nautilus.render()

        for b in buoys:
            glPushMatrix()
            glTranslatef(b.position.x, b.position.y, b.position.z)
            b.mesh.render()
            glPopMatrix()

        glLoadIdentity()
        glTranslatef(0, 0, -near * 1.01)

        glDisable(GL_DEPTH_TEST)
        glDisable(GL_LIGHTING)
        glDisable(GL_CULL_FACE)

        if buoys:
            buoy = buoys[0]
        elif level == 1:
            show_level = 300
            level += 1
            level2(meshes)
        elif level == 2:
            level += 1

        nautilus.buoys_left = len(buoys)
        gauges.render_compass(nautilus, buoy, camera)
        gauges.render_pitch(nautilus, buoy, camera)
        gauges.render_horizon(nautilus, camera)
        gauges.render_rpm(nautilus, camera)
        gauges.render_altimeter(nautilus, camera)
        gauges.render_air(nautilus, camera)
        gauges.render_score(nautilus, camera)
        if show_help: gauges.render_help(nautilus, camera)
        if show_level:
            gauges.render_level(level,
                "Collect the orange air tanks in the right order!", camera)
        if level == 3:
            gauges.render_level(level,
                "Sorry, only level 1 and 2 are done!", camera)

        pygame.display.flip()

        t = pygame.time.get_ticks()
        pygame.time.wait(next - t)
        t = next
        next = t + 1000 / FPS

if __name__ == "__main__":
    main()
